# -*- coding: utf-8 -*-
"""Cisco Catalyst Center Health and Performance API wrapper.

Copyright (c) 2026 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from builtins import *  # noqa: F401,F403


from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class HealthAndPerformance(object):
    """Cisco Catalyst Center Health and Performance API (version: 3.1.6.0).

    Wraps the Catalyst Center Health and Performance
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new HealthAndPerformance
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Catalyst Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(HealthAndPerformance, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def retrieves_diagnostic_task_by_id(self, id, headers=None, **request_parameters):
        """This API retrieves the diagnostic task identified by the specified `id`.

        Args:
            id(str): id path parameter. The `id` of the diagnostic task to be retrieved.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-diagnostic-task-by-i-d
        """
        check_type(headers, dict)
        check_type(id, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            "id": id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticTasks/{id}"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_f1144e25e659d1aedcfe02afca97cc_v3_1_6_0", json_data
        )

    def retrieves_diagnostic_task_details_by_id(
        self, id, headers=None, **request_parameters
    ):
        """This API retrieves the details of the diagnostic task identified by the specified `id`.

        Args:
            id(str): id path parameter. The `id` of the diagnostic task to be retrieved.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-diagnostic-task-details-by-i-d
        """
        check_type(headers, dict)
        check_type(id, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            "id": id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticTasks/{id}/detail"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_a656639f78625002805a9ad1257f9cde_v3_1_6_0", json_data
        )

    def retrieves_all_the_validation_sets(
        self, view=None, headers=None, **request_parameters
    ):
        """Retrieves all the validation sets and optionally the contained validations.

        Args:
            view(str): view query parameter. When the query parameter `view=DETAIL` is passed, all validation sets
                and associated validations will be returned. When the query parameter `view=DEFAULT` is
                passed, only validation sets metadata will be returned.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-all-the-validation-sets
        """
        check_type(headers, dict)
        check_type(view, str)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "view": view,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationSets"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_d6fc1397d48d52449923716aff009d3c_v3_1_6_0", json_data
        )

    def retrieves_validation_details_for_a_validation_set(
        self, id, headers=None, **request_parameters
    ):
        """Retrieves validation details for the given validation set id.

        Args:
            id(str): id path parameter. Validation set id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-validation-details-for-a-validation-set
        """
        check_type(headers, dict)
        check_type(id, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            "id": id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationSets/{id}"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_d95307fdbf5b169d9d05e3151f61ac_v3_1_6_0", json_data
        )

    def retrieves_the_list_of_validation_workflows(
        self,
        end_time=None,
        limit=None,
        offset=None,
        run_status=None,
        start_time=None,
        headers=None,
        **request_parameters
    ):
        """Retrieves the workflows that have been successfully submitted and are currently available. This is sorted by
        `submitTime`.

        Args:
            start_time(int): startTime query parameter. Workflows started after the given time (as milliseconds
                since UNIX epoch).
            end_time(int): endTime query parameter. Workflows started before the given time (as milliseconds since
                UNIX epoch).
            run_status(str): runStatus query parameter. Execution status of the workflow. If the workflow is
                successfully submitted, runStatus is `PENDING`. If the workflow execution has started,
                runStatus is `IN_PROGRESS`. If the workflow executed is completed with all validations
                executed, runStatus is `COMPLETED`. If the workflow execution fails while running
                validations, runStatus is `FAILED`.
            offset(int): offset query parameter. The first record to show for this page; the first record is
                numbered 1.
            limit(int): limit query parameter. Specifies the maximum number of workflows to return per page. Must be
                an integer between 1 and 500, inclusive.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-list-of-validation-workflows
        """
        check_type(headers, dict)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(run_status, str)
        check_type(offset, int)
        check_type(limit, int)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "startTime": start_time,
            "endTime": end_time,
            "runStatus": run_status,
            "offset": offset,
            "limit": limit,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationWorkflows"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_a53d325f85e5549b7c5957c6ecbd891_v3_1_6_0", json_data
        )

    def submits_the_workflow_for_executing_validations(
        self,
        description=None,
        name=None,
        validationSetIds=None,
        headers=None,
        payload=None,
        active_validation=True,
        **request_parameters
    ):
        """Submits the workflow for executing the validations for the given validation specifications.

        Args:
            description(string): Health and Performance's Description of the workflow to run.
            name(string): Health and Performance's Name of the workflow to run. It must be unique.
            validationSetIds(list): Health and Performance's List of validation set ids (list of strings).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!submits-the-workflow-for-executing-validations
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if "Content-Type" in headers:
                check_type(headers.get("Content-Type"), str, may_be_none=False)
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}
        _payload = {
            "name": name,
            "description": description,
            "validationSetIds": validationSetIds,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator(
                "jsd_cf9d39cef5e95bb9bd48d5f86e094c99_v3_1_6_0"
            ).validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationWorkflows"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload, headers=_headers
            )
        else:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload
            )

        return self._object_factory(
            "bpm_cf9d39cef5e95bb9bd48d5f86e094c99_v3_1_6_0", json_data
        )

    def retrieves_the_count_of_validation_workflows(
        self,
        end_time=None,
        run_status=None,
        start_time=None,
        headers=None,
        **request_parameters
    ):
        """Retrieves the count of workflows that have been successfully submitted and are currently available.

        Args:
            start_time(int): startTime query parameter. Workflows started after the given time (as milliseconds
                since UNIX epoch).
            end_time(int): endTime query parameter. Workflows started before the given time (as milliseconds since
                UNIX epoch).
            run_status(str): runStatus query parameter. Execution status of the workflow. If the workflow is
                successfully submitted, runStatus is `PENDING`. If the workflow execution has started,
                runStatus is `IN_PROGRESS`. If the workflow executed is completed with all validations
                executed, runStatus is `COMPLETED`. If the workflow execution fails while running
                validations, runStatus is `FAILED`.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-the-count-of-validation-workflows
        """
        check_type(headers, dict)
        check_type(start_time, int)
        check_type(end_time, int)
        check_type(run_status, str)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "startTime": start_time,
            "endTime": end_time,
            "runStatus": run_status,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationWorkflows/count"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_b174a2fc5171520d9423c9a50f7394e7_v3_1_6_0", json_data
        )

    def deletes_a_validation_workflow(self, id, headers=None, **request_parameters):
        """Deletes the workflow for the given id.

        Args:
            id(str): id path parameter. Workflow id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!deletes-a-validation-workflow
        """
        check_type(headers, dict)
        check_type(id, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            "id": id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationWorkflows/{id}"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_b3ab76a74dae51fabf39b2ad85c3c58f_v3_1_6_0", json_data
        )

    def retrieves_validation_workflow_details(
        self, id, headers=None, **request_parameters
    ):
        """Retrieves workflow details for a workflow id.

        Args:
            id(str): id path parameter. Workflow id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!retrieves-validation-workflow-details
        """
        check_type(headers, dict)
        check_type(id, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            "id": id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnosticValidationWorkflows/{id}"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_c36c30b8c5ddfbf9ccf36db5dd68a_v3_1_6_0", json_data
        )

    def system_health(
        self,
        domain=None,
        limit=None,
        offset=None,
        subdomain=None,
        summary=None,
        headers=None,
        **request_parameters
    ):
        """This API retrieves the latest system events.

        Args:
            summary(bool): summary query parameter. Fetch the latest high severity event.
            domain(str): domain query parameter. Fetch system events with this domain. Possible values of domain are
                listed here : /dna/platform/app/consumer-portal/developer-toolkit/events.
            subdomain(str): subdomain query parameter. Fetch system events with this subdomain. Possible values of
                subdomain are listed here : /dna/platform/app/consumer-portal/developer-toolkit/events.
            limit(int): limit query parameter. Specifies the maximum number of system health events to return per
                page. Must be an integer between 1 and 50, inclusive.
            offset(int): offset query parameter. Specifies the starting point for the list of system health events
                to return. Must be an integer greater than or equal to 0.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-health-a-p-i
        """
        check_type(headers, dict)
        check_type(summary, bool)
        check_type(domain, str)
        check_type(subdomain, str)
        check_type(limit, int)
        check_type(offset, int)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "summary": summary,
            "domain": domain,
            "subdomain": subdomain,
            "limit": limit,
            "offset": offset,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnostics/system/health"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_d0acccfae6885bc28f8f39c67f4acfc1_v3_1_6_0", json_data
        )

    def system_health_count(
        self, domain=None, subdomain=None, headers=None, **request_parameters
    ):
        """This API gives the count of the latest system events.

        Args:
            domain(str): domain query parameter. Fetch system events with this domain. Possible values of domain are
                listed here : /dna/platform/app/consumer-portal/developer-toolkit/events.
            subdomain(str): subdomain query parameter. Fetch system events with this subdomain. Possible values of
                subdomain are listed here : /dna/platform/app/consumer-portal/developer-toolkit/events.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-health-count-a-p-i
        """
        check_type(headers, dict)
        check_type(domain, str)
        check_type(subdomain, str)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "domain": domain,
            "subdomain": subdomain,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnostics/system/health/count"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_f6dd603bc35db1948f31c782a37647_v3_1_6_0", json_data
        )

    def system_performance(
        self,
        end_time=None,
        function=None,
        kpi=None,
        start_time=None,
        headers=None,
        **request_parameters
    ):
        """Retrieves the average values of cluster key performance indicators (KPIs), such as CPU utilization, memory
        utilization or network rates over the past 15 minutes. Query parameters 'function', 'startTime' and
        'endTime' are no longer supported.

        Args:
            kpi(str): kpi query parameter. Valid values: cpu,memory,network.
            function(str): function query parameter. In this release this field has been deprecated and no longer
                supported. Previously supported functions were 'sum', 'average', and 'max'. Now, only
                the last 15 minutes average of cluster key performance indicators (KPIs) is returned,
                regardless of the specified function. For example, if 'sum', 'average', or 'max' is
                provided, it will be ignored and the 'average' of the last 15 minutes will be returned.
            start_time(int): startTime query parameter. In this release this field has been deprecated and no longer
                supported. Only the last 15 minutes average of cluster key performance indicators (KPIs)
                is returned, regardless of the provided epoch time. For example, if any epoch time is
                specified for this field, it will be ignored and 15 minutes before the current time will
                be considered.
            end_time(int): endTime query parameter. In this release this field has been deprecated and no longer
                supported. Only the last 15 minutes average of cluster key performance indicators (KPIs)
                is returned, regardless of the provided epoch time. For example, if any epoch time is
                specified for this field, it will be ignored and the current time will be considered.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-performance-a-p-i
        """
        check_type(headers, dict)
        check_type(kpi, str)
        check_type(function, str)
        check_type(start_time, int)
        check_type(end_time, int)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "kpi": kpi,
            "function": function,
            "startTime": start_time,
            "endTime": end_time,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnostics/system/performance"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_cfcb7a875f215cb4ba59be38abb871e6_v3_1_6_0", json_data
        )

    def system_performance_historical(
        self,
        end_time=None,
        kpi=None,
        start_time=None,
        headers=None,
        **request_parameters
    ):
        """Retrieves hourly data of cluster key performance indicators (KPIs), like CPU utilization, memory utilization or
        network rates for the past 24 hours. Query parameters 'startTime' and 'endTime' are no longer supported.

        Args:
            kpi(str): kpi query parameter. Fetch historical data for this kpi. Valid values: cpu,memory,network.
            start_time(int): startTime query parameter. In this release this field has been deprecated and no longer
                supported. This is restricted to a 24-hour time range with hourly samples, regardless of
                the provided epoch time.  For example, if any epoch time is specified for this field, it
                will be ignored and 24 hours before the current time will be considered.
            end_time(int): endTime query parameter. In this release this field has been deprecated and no longer
                supported. This is restricted to a 24-hour time range with hourly samples, regardless of
                the provided epoch time.  For example, if any epoch time is specified for this field, it
                will be ignored and the current time will be considered.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!system-performance-historical-a-p-i
        """
        check_type(headers, dict)
        check_type(kpi, str)
        check_type(start_time, int)
        check_type(end_time, int)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "kpi": kpi,
            "startTime": start_time,
            "endTime": end_time,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/intent/api/v1/diagnostics/system/performance/histor" + "y"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_f131d712dc253dca528c0298b3e41c6_v3_1_6_0", json_data
        )


# Alias Functions



# -*- coding: utf-8 -*-
"""Cisco Catalyst Center System Software Upgrade API wrapper.

Copyright (c) 2026 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from builtins import *  # noqa: F401,F403


from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class SystemSoftwareUpgrade(object):
    """Cisco Catalyst Center System Software Upgrade API (version: 3.1.6.0).

    Wraps the Catalyst Center System Software Upgrade
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new SystemSoftwareUpgrade
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Catalyst Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(SystemSoftwareUpgrade, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def delete_downloaded_release(
        self, release_name, release_version, headers=None, **request_parameters
    ):
        """This api is used to trigger the workflow to delete the downloaded release.

        Args:
            release_name(str): releaseName query parameter. The releaseName of the downloaded release to be deleted.
            release_version(str): releaseVersion query parameter. The releaseVersion of the downloaded release to be
                deleted.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!delete-downloaded-release
        """
        check_type(headers, dict)
        check_type(release_name, str, may_be_none=False)
        check_type(release_version, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "releaseName": release_name,
            "releaseVersion": release_version,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/downloadSoftwareRelease"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_b57f9a355d58e4a951d3db3f293_v3_1_6_0", json_data
        )

    def download_release(
        self,
        optionalPackages=None,
        releaseName=None,
        releaseVersion=None,
        headers=None,
        payload=None,
        active_validation=True,
        **request_parameters
    ):
        """This api is used to trigger download workflow of a specific release.

        Args:
            optionalPackages(list): System Software Upgrade's Provide the list of optional package's id to be
                downloaded.Obtain the `id` from the `packages[n].id` attribute where
                `packages[n].optional` is true from the `/dna/system/api/v1/releases/releaseSummary` API
                where the `releaseName` and `releaseVersion` is the requested download version. (list of
                strings).
            releaseName(string): System Software Upgrade's The `releaseName` of the release to be downloaded.Obtain
                the releaseName from the `name` attribute in the response of the
                `/dna/system/api/v1/releases` API.
            releaseVersion(string): System Software Upgrade's The `releaseVersion` of the release to be
                downloaded.Obtain the releaseVersion from the `version` attribute in the response of the
                `/dna/system/api/v1/releases` API.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!download-release
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if "Content-Type" in headers:
                check_type(headers.get("Content-Type"), str, may_be_none=False)
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}
        _payload = {
            "optionalPackages": optionalPackages,
            "releaseName": releaseName,
            "releaseVersion": releaseVersion,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator(
                "jsd_f57dc44184564cb97a6573a44ad394_v3_1_6_0"
            ).validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/downloadSoftwareRelease"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload, headers=_headers
            )
        else:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload
            )

        return self._object_factory(
            "bpm_f57dc44184564cb97a6573a44ad394_v3_1_6_0", json_data
        )

    def update_downloaded_release(
        self,
        optionalPackages=None,
        releaseName=None,
        releaseVersion=None,
        headers=None,
        payload=None,
        active_validation=True,
        **request_parameters
    ):
        """After the specified release has been downloaded successfully, users can download additional optional packages
        using this API.

        Args:
            optionalPackages(list): System Software Upgrade's Provide the list of optional package's id to be
                downloaded.Obtain the `id` from the `packages[n].id` attribute where
                `packages[n].optional` is true from the `/dna/system/api/v1/releases/releaseSummary` API
                where the `releaseName` and `releaseVersion` is the requested download version. (list of
                strings).
            releaseName(string): System Software Upgrade's The `releaseName` of the downloaded release to be
                updated.
            releaseVersion(string): System Software Upgrade's The `releaseVersion` of the downloaded release to be
                updated.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!update-downloaded-release
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if "Content-Type" in headers:
                check_type(headers.get("Content-Type"), str, may_be_none=False)
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}
        _payload = {
            "optionalPackages": optionalPackages,
            "releaseName": releaseName,
            "releaseVersion": releaseVersion,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator(
                "jsd_e4ddf9b60efb59ac8f6a200a563d9d72_v3_1_6_0"
            ).validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/downloadSoftwareRelease"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(
                endpoint_full_url, params=_params, json=_payload, headers=_headers
            )
        else:
            json_data = self._session.put(
                endpoint_full_url, params=_params, json=_payload
            )

        return self._object_factory(
            "bpm_e4ddf9b60efb59ac8f6a200a563d9d72_v3_1_6_0", json_data
        )

    def install_optional_packages(
        self,
        optionalPackages=None,
        headers=None,
        payload=None,
        active_validation=True,
        **request_parameters
    ):
        """This API is used to trigger the workflow for installing optional packages.

        Args:
            optionalPackages(list): System Software Upgrade's Provide the list of optional package's id to be
                installed .Use the /dna/system/api/v1/releases/releaseSummary API to obtain the optional
                package IDs. The `releaseName` and `releaseVersion` should correspond to the installed
                release name and version, which can be obtained from the `name` and `version` attributes
                in the response of the `/dna/system/api/v1/installedRelease` API. In the releaseSummary
                API response, optional packages can be identified by the attribute
                `packages[n].optional` is true. Provide the `packages[n].id` of these optional packages.
                (list of strings).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!install-optional-packages
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if "Content-Type" in headers:
                check_type(headers.get("Content-Type"), str, may_be_none=False)
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}
        _payload = {
            "optionalPackages": optionalPackages,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator(
                "jsd_f2dc0b2dd265d3bb69c5aeb5cff2f13_v3_1_6_0"
            ).validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/installOptionalPackages"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload, headers=_headers
            )
        else:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload
            )

        return self._object_factory(
            "bpm_f2dc0b2dd265d3bb69c5aeb5cff2f13_v3_1_6_0", json_data
        )

    def get_installed_release_details(self, headers=None, **request_parameters):
        """This api is used to get the installed release details.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-installed-release-details
        """
        check_type(headers, dict)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/installedRelease"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_af1fd1bd1557b880106f71c5db9f66_v3_1_6_0", json_data
        )

    def get_all_release(self, headers=None, **request_parameters):
        """This api is used to get all available release.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-all-release
        """
        check_type(headers, dict)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/releases"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_fc2a12b907a35a92a47b7877b5207c4d_v3_1_6_0", json_data
        )

    def get_release_detail(
        self, release_name, release_version, headers=None, **request_parameters
    ):
        """This API is used to retrieve a specific release and the list of packages available in that release.

        Args:
            release_name(str): releaseName query parameter. The releaseName of the release to be retrieved.
            release_version(str): releaseVersion query parameter. The releaseVersion of the release to be retrieved.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-release-detail
        """
        check_type(headers, dict)
        check_type(release_name, str, may_be_none=False)
        check_type(release_version, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {
            "releaseName": release_name,
            "releaseVersion": release_version,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/releases/releaseSummary"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_a0c33ecb0a475f55b3518736d8a9c601_v3_1_6_0", json_data
        )

    def get_software_management_execution_details(
        self, id, headers=None, **request_parameters
    ):
        """This api is used to get execution status and task details of a specific software management workflows.

        Args:
            id(str): id path parameter. The id of the execution detail to be retrieved.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!get-software-management-execution-details
        """
        check_type(headers, dict)
        check_type(id, str, may_be_none=False)
        if headers is not None:
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            "id": id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/softwareManagementExecutions/{id}"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(
                endpoint_full_url, params=_params, headers=_headers
            )
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory(
            "bpm_fc20a7cd31655bf1ae775e4bf645a9e1_v3_1_6_0", json_data
        )

    def uninstall_optional_packages(
        self,
        optionalPackages=None,
        headers=None,
        payload=None,
        active_validation=True,
        **request_parameters
    ):
        """This API is used to trigger the workflow for uninstalling optional packages.

        Args:
            optionalPackages(list): System Software Upgrade's Provide the list of optional package's id to be
                uninstalled. Use the `/dna/system/api/v1/installedRelease` API to obtain the optional
                package IDs. In the installedRelease API response, installed optional packages can be
                identified by the attributes `packages[n].optional` is true. and `packages[n].status` is
                DEPLOYED. Provide the `packages[n].id` of these optional packages. (list of strings).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!uninstall-optional-packages
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if "Content-Type" in headers:
                check_type(headers.get("Content-Type"), str, may_be_none=False)
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}
        _payload = {
            "optionalPackages": optionalPackages,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator(
                "jsd_d0456ad1d5bda99f7d9254f8a1ec3_v3_1_6_0"
            ).validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/uninstallOptionalPackages"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload, headers=_headers
            )
        else:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload
            )

        return self._object_factory(
            "bpm_d0456ad1d5bda99f7d9254f8a1ec3_v3_1_6_0", json_data
        )

    def upgrade_release(
        self,
        optionalPackages=None,
        releaseName=None,
        releaseVersion=None,
        headers=None,
        payload=None,
        active_validation=True,
        **request_parameters
    ):
        """This api is used to trigger the workflow to upgrade the downloaded release.

        Args:
            optionalPackages(list): System Software Upgrade's Provide the list of optional package's id to be
                downloaded and upgraded . Use the `/dna/system/api/v1/releases/releaseSummary` API to
                obtain the optional package IDs. The `releaseName` and `releaseVersion` should
                correspond to the downloaded release.In the releases summary API response, optional
                packages can be identified by the attribute `packages[n].optional` is true. Provide the
                `packages[n].id` of these optional packages. (list of strings).
            releaseName(string): System Software Upgrade's The `releaseName` of the downloaded release to be
                upgraded.
            releaseVersion(string): System Software Upgrade's The `releaseVersion` of the downloaded release to be
                upgraded.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Catalyst Center cloud returns an error.
        Documentation Link:
            https://developer.cisco.com/docs/dna-center/#!upgrade-release
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if "Content-Type" in headers:
                check_type(headers.get("Content-Type"), str, may_be_none=False)
            if "X-Auth-Token" in headers:
                check_type(headers.get("X-Auth-Token"), str, may_be_none=False)

        _params = {}
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {}
        _payload = {
            "optionalPackages": optionalPackages,
            "releaseName": releaseName,
            "releaseVersion": releaseVersion,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator(
                "jsd_c90586f562b8fc0451b7e71b35a_v3_1_6_0"
            ).validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = "/dna/system/api/v1/upgradeSoftwareRelease"
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload, headers=_headers
            )
        else:
            json_data = self._session.post(
                endpoint_full_url, params=_params, json=_payload
            )

        return self._object_factory(
            "bpm_c90586f562b8fc0451b7e71b35a_v3_1_6_0", json_data
        )


# Alias Functions



# -*- coding: utf-8 -*-
"""Cisco Catalyst Center ImportsTheTemplatesProvided data model.

Copyright (c) 2025 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import json
from builtins import *

import fastjsonschema

from catalystcentersdk.exceptions import MalformedRequest


class JSONSchemaValidatorDb7B6C4F0542AAb9FE7Cf5C995F83(object):
    """ImportsTheTemplatesProvided request schema definition."""
    def __init__(self):
        super(JSONSchemaValidatorDb7B6C4F0542AAb9FE7Cf5C995F83, self).__init__()
        self._validator = fastjsonschema.compile(json.loads('''
{
    "$schema": "http://json-schema.org/draft-04/schema#",
    "items": {
        "properties": {
            "author": {
                "type": "string"
            },
            "composite": {
                "type": "boolean"
            },
            "containingTemplates": {
                "items": {
                    "properties": {
                        "composite": {
                            "type": "boolean"
                        },
                        "description": {
                            "type": "string"
                        },
                        "deviceTypes": {
                            "items": {
                                "properties": {
                                    "productFamily": {
                                        "type": "string"
                                    },
                                    "productSeries": {
                                        "type": "string"
                                    },
                                    "productType": {
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "productFamily"
                                ],
                                "type": "object"
                            },
                            "type": "array"
                        },
                        "id": {
                            "type": "string"
                        },
                        "language": {
                            "type": "string"
                        },
                        "name": {
                            "type": "string"
                        },
                        "projectName": {
                            "type": "string"
                        },
                        "rollbackTemplateParams": {
                            "items": {
                                "properties": {
                                    "binding": {
                                        "type": "string"
                                    },
                                    "customOrder": {
                                        "type": "integer"
                                    },
                                    "dataType": {
                                        "enum": [
                                            "STRING",
                                            "INTEGER",
                                            "IPADDRESS",
                                            "MACADDRESS",
                                            "SECTIONDIVIDER"
                                        ],
                                        "type": "string"
                                    },
                                    "defaultValue": {
                                        "type": "string"
                                    },
                                    "description": {
                                        "type": "string"
                                    },
                                    "displayName": {
                                        "type": "string"
                                    },
                                    "group": {
                                        "type": "string"
                                    },
                                    "id": {
                                        "type": "string"
                                    },
                                    "instructionText": {
                                        "type": "string"
                                    },
                                    "key": {
                                        "type": "string"
                                    },
                                    "notParam": {
                                        "type": "boolean"
                                    },
                                    "order": {
                                        "type": "integer"
                                    },
                                    "paramArray": {
                                        "type": "boolean"
                                    },
                                    "parameterName": {
                                        "type": "string"
                                    },
                                    "provider": {
                                        "type": "string"
                                    },
                                    "range": {
                                        "items": {
                                            "properties": {
                                                "id": {
                                                    "type": "string"
                                                },
                                                "maxValue": {
                                                    "type": "integer"
                                                },
                                                "minValue": {
                                                    "type": "integer"
                                                }
                                            },
                                            "required": [
                                                "maxValue",
                                                "minValue"
                                            ],
                                            "type": "object"
                                        },
                                        "type": "array"
                                    },
                                    "required": {
                                        "type": "boolean"
                                    },
                                    "selection": {
                                        "properties": {
                                            "defaultSelectedValues": {
                                                "items": {
                                                    "type": "string"
                                                },
                                                "type": "array"
                                            },
                                            "id": {
                                                "type": "string"
                                            },
                                            "selectionType": {
                                                "enum": [
                                                    "SINGLE_SELECT",
                                                    "MULTI_SELECT"
                                                ],
                                                "type": "string"
                                            },
                                            "selectionValues": {
                                                "type": "object"
                                            }
                                        },
                                        "required": [
                                            "selectionType",
                                            "selectionValues"
                                        ],
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "dataType",
                                    "parameterName"
                                ],
                                "type": "object"
                            },
                            "type": "array"
                        },
                        "tags": {
                            "items": {
                                "properties": {
                                    "id": {
                                        "type": "string"
                                    },
                                    "name": {
                                        "type": "string"
                                    }
                                },
                                "required": [
                                    "name"
                                ],
                                "type": "object"
                            },
                            "type": "array"
                        },
                        "templateContent": {
                            "type": "string"
                        },
                        "templateParams": {
                            "items": {
                                "properties": {
                                    "binding": {
                                        "type": "string"
                                    },
                                    "customOrder": {
                                        "type": "integer"
                                    },
                                    "dataType": {
                                        "enum": [
                                            "STRING",
                                            "INTEGER",
                                            "IPADDRESS",
                                            "MACADDRESS",
                                            "SECTIONDIVIDER"
                                        ],
                                        "type": "string"
                                    },
                                    "defaultValue": {
                                        "type": "string"
                                    },
                                    "description": {
                                        "type": "string"
                                    },
                                    "displayName": {
                                        "type": "string"
                                    },
                                    "group": {
                                        "type": "string"
                                    },
                                    "id": {
                                        "type": "string"
                                    },
                                    "instructionText": {
                                        "type": "string"
                                    },
                                    "key": {
                                        "type": "string"
                                    },
                                    "notParam": {
                                        "type": "boolean"
                                    },
                                    "order": {
                                        "type": "integer"
                                    },
                                    "paramArray": {
                                        "type": "boolean"
                                    },
                                    "parameterName": {
                                        "type": "string"
                                    },
                                    "provider": {
                                        "type": "string"
                                    },
                                    "range": {
                                        "items": {
                                            "properties": {
                                                "id": {
                                                    "type": "string"
                                                },
                                                "maxValue": {
                                                    "type": "integer"
                                                },
                                                "minValue": {
                                                    "type": "integer"
                                                }
                                            },
                                            "required": [
                                                "maxValue",
                                                "minValue"
                                            ],
                                            "type": "object"
                                        },
                                        "type": "array"
                                    },
                                    "required": {
                                        "type": "boolean"
                                    },
                                    "selection": {
                                        "properties": {
                                            "defaultSelectedValues": {
                                                "items": {
                                                    "type": "string"
                                                },
                                                "type": "array"
                                            },
                                            "id": {
                                                "type": "string"
                                            },
                                            "selectionType": {
                                                "enum": [
                                                    "SINGLE_SELECT",
                                                    "MULTI_SELECT"
                                                ],
                                                "type": "string"
                                            },
                                            "selectionValues": {
                                                "type": "object"
                                            }
                                        },
                                        "required": [
                                            "selectionType",
                                            "selectionValues"
                                        ],
                                        "type": "object"
                                    }
                                },
                                "required": [
                                    "dataType",
                                    "parameterName"
                                ],
                                "type": "object"
                            },
                            "type": "array"
                        },
                        "version": {
                            "type": "string"
                        }
                    },
                    "required": [
                        "deviceTypes",
                        "language",
                        "name",
                        "projectName"
                    ],
                    "type": "object"
                },
                "type": "array"
            },
            "createTime": {
                "type": "integer"
            },
            "customParamsOrder": {
                "type": "boolean"
            },
            "description": {
                "type": "string"
            },
            "deviceTypes": {
                "items": {
                    "properties": {
                        "productFamily": {
                            "type": "string"
                        },
                        "productSeries": {
                            "type": "string"
                        },
                        "productType": {
                            "type": "string"
                        }
                    },
                    "required": [
                        "productFamily"
                    ],
                    "type": "object"
                },
                "type": "array"
            },
            "failurePolicy": {
                "enum": [
                    "ABORT_ON_ERROR",
                    "CONTINUE_ON_ERROR",
                    "ROLLBACK_ON_ERROR",
                    "ROLLBACK_TARGET_ON_ERROR",
                    "ABORT_TARGET_ON_ERROR"
                ],
                "type": "string"
            },
            "id": {
                "type": "string"
            },
            "language": {
                "type": "string"
            },
            "lastUpdateTime": {
                "type": "integer"
            },
            "latestVersionTime": {
                "type": "integer"
            },
            "name": {
                "type": "string"
            },
            "parentTemplateId": {
                "type": "string"
            },
            "projectId": {
                "type": "string"
            },
            "projectName": {
                "type": "string"
            },
            "rollbackTemplateContent": {
                "type": "string"
            },
            "rollbackTemplateParams": {
                "items": {
                    "properties": {
                        "binding": {
                            "type": "string"
                        },
                        "customOrder": {
                            "type": "integer"
                        },
                        "dataType": {
                            "enum": [
                                "STRING",
                                "INTEGER",
                                "IPADDRESS",
                                "MACADDRESS",
                                "SECTIONDIVIDER"
                            ],
                            "type": "string"
                        },
                        "defaultValue": {
                            "type": "string"
                        },
                        "description": {
                            "type": "string"
                        },
                        "displayName": {
                            "type": "string"
                        },
                        "group": {
                            "type": "string"
                        },
                        "id": {
                            "type": "string"
                        },
                        "instructionText": {
                            "type": "string"
                        },
                        "key": {
                            "type": "string"
                        },
                        "notParam": {
                            "type": "boolean"
                        },
                        "order": {
                            "type": "integer"
                        },
                        "paramArray": {
                            "type": "boolean"
                        },
                        "parameterName": {
                            "type": "string"
                        },
                        "provider": {
                            "type": "string"
                        },
                        "range": {
                            "items": {
                                "properties": {
                                    "id": {
                                        "type": "string"
                                    },
                                    "maxValue": {
                                        "type": "integer"
                                    },
                                    "minValue": {
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "maxValue",
                                    "minValue"
                                ],
                                "type": "object"
                            },
                            "type": "array"
                        },
                        "required": {
                            "type": "boolean"
                        },
                        "selection": {
                            "properties": {
                                "defaultSelectedValues": {
                                    "items": {
                                        "type": "string"
                                    },
                                    "type": "array"
                                },
                                "id": {
                                    "type": "string"
                                },
                                "selectionType": {
                                    "enum": [
                                        "SINGLE_SELECT",
                                        "MULTI_SELECT"
                                    ],
                                    "type": "string"
                                },
                                "selectionValues": {
                                    "type": "object"
                                }
                            },
                            "required": [
                                "selectionType",
                                "selectionValues"
                            ],
                            "type": "object"
                        }
                    },
                    "required": [
                        "dataType",
                        "parameterName"
                    ],
                    "type": "object"
                },
                "type": "array"
            },
            "softwareType": {
                "type": "string"
            },
            "softwareVariant": {
                "type": "string"
            },
            "softwareVersion": {
                "type": "string"
            },
            "tags": {
                "items": {
                    "properties": {
                        "id": {
                            "type": "string"
                        },
                        "name": {
                            "type": "string"
                        }
                    },
                    "required": [
                        "name"
                    ],
                    "type": "object"
                },
                "type": "array"
            },
            "templateContent": {
                "type": "string"
            },
            "templateParams": {
                "items": {
                    "properties": {
                        "binding": {
                            "type": "string"
                        },
                        "customOrder": {
                            "type": "integer"
                        },
                        "dataType": {
                            "enum": [
                                "STRING",
                                "INTEGER",
                                "IPADDRESS",
                                "MACADDRESS",
                                "SECTIONDIVIDER"
                            ],
                            "type": "string"
                        },
                        "defaultValue": {
                            "type": "string"
                        },
                        "description": {
                            "type": "string"
                        },
                        "displayName": {
                            "type": "string"
                        },
                        "group": {
                            "type": "string"
                        },
                        "id": {
                            "type": "string"
                        },
                        "instructionText": {
                            "type": "string"
                        },
                        "key": {
                            "type": "string"
                        },
                        "notParam": {
                            "type": "boolean"
                        },
                        "order": {
                            "type": "integer"
                        },
                        "paramArray": {
                            "type": "boolean"
                        },
                        "parameterName": {
                            "type": "string"
                        },
                        "provider": {
                            "type": "string"
                        },
                        "range": {
                            "items": {
                                "properties": {
                                    "id": {
                                        "type": "string"
                                    },
                                    "maxValue": {
                                        "type": "integer"
                                    },
                                    "minValue": {
                                        "type": "integer"
                                    }
                                },
                                "required": [
                                    "maxValue",
                                    "minValue"
                                ],
                                "type": "object"
                            },
                            "type": "array"
                        },
                        "required": {
                            "type": "boolean"
                        },
                        "selection": {
                            "properties": {
                                "defaultSelectedValues": {
                                    "items": {
                                        "type": "string"
                                    },
                                    "type": "array"
                                },
                                "id": {
                                    "type": "string"
                                },
                                "selectionType": {
                                    "enum": [
                                        "SINGLE_SELECT",
                                        "MULTI_SELECT"
                                    ],
                                    "type": "string"
                                },
                                "selectionValues": {
                                    "type": "object"
                                }
                            },
                            "required": [
                                "selectionType",
                                "selectionValues"
                            ],
                            "type": "object"
                        }
                    },
                    "required": [
                        "dataType",
                        "parameterName"
                    ],
                    "type": "object"
                },
                "type": "array"
            },
            "validationErrors": {
                "properties": {
                    "rollbackTemplateErrors": {},
                    "templateErrors": {},
                    "templateId": {
                        "type": "string"
                    },
                    "templateVersion": {
                        "type": "string"
                    }
                },
                "type": "object"
            },
            "version": {
                "type": "string"
            }
        },
        "required": [
            "deviceTypes",
            "language",
            "name",
            "projectName",
            "softwareType"
        ],
        "type": "object"
    },
    "type": "array"
}
'''))

    def validate(self, request):
        try:
            self._validator(request)
        except fastjsonschema.exceptions.JsonSchemaException as e:
            raise MalformedRequest(
                '{} is invalid. Reason: {}'.format(request, e.message)
            )
